@@ -1,5 +1,5 @@
 #ifndef lint
-static  char sccsid[] = "@(#)bounce.c 1.3 88/02/26 XLOCK";
+static  char sccsid[] = "@(#)bounce.c	2.7 95/02/21 xlockmore";
 #endif
 
 /*-
@@ -156,25 +156,25 @@ initbounce(win)
         bp->balls[i].size = 5;
       else
         bp->balls[i].size = (logo[0].width + logo[0].height) / 2;
-      bp->balls[i].vx = ((random() % 2) ? -1 : 1) *
-        (random() % MAX_STRENGTH + 1);
+      bp->balls[i].vx = ((RAND() % 2) ? -1 : 1) *
+        (RAND() % MAX_STRENGTH + 1);
       bp->balls[i].x = (bp->balls[i].vx >= 0) ?
         0 : bp->width - bp->balls[i].size;
-      bp->balls[i].y = random() % (bp->height / 2);
+      bp->balls[i].y = RAND() % (bp->height / 2);
       if (i == collide(i)) {
         if (!mono && Scr[screen].npixels > 2)
           bp->balls[i].color =
-            Scr[screen].pixels[random() % Scr[screen].npixels];
+            Scr[screen].pixels[RAND() % Scr[screen].npixels];
         else
           bp->balls[i].color = WhitePixel(dsp, screen);
         bp->balls[i].xlast = -1;
         bp->balls[i].ylast = 0;
         bp->balls[i].spincount = 1;
         bp->balls[i].spindelay = 1;
-        bp->balls[i].vy = ((random() % 2) ? -1 : 1) * (random() % MAX_STRENGTH);
+        bp->balls[i].vy = ((RAND() % 2) ? -1 : 1) * (RAND() % MAX_STRENGTH);
         bp->balls[i].spindir = 0;
         bp->balls[i].vang = 0;
-        bp->balls[i].orient = random() % ORIENTS;
+        bp->balls[i].orient = RAND() % ORIENTS;
         i++;
       } else
         bp->nballs--;
@@ -247,7 +247,7 @@ drawbounce(win)
        }
        for (i = 0; i < bp->nballs; i++)
          checkCollision(i);
-       if (!(random() % TIME)) /* Put some randomness into the time */
+       if (!(RAND() % TIME)) /* Put some randomness into the time */
          restartnum--;
        if (!restartnum) {
 	 initbounce(win);
