@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)helix.c 1.3 92/01/01 XLOCK";
+static char sccsid[] = "@(#)helix.c	2.7 95/02/21 xlockmore";
 #endif
 /*
  * String art
@@ -142,9 +142,9 @@ random_helix (win)
   factor3 = 2;
   factor4 = 2;
 
-  divisor = ((random() / MAXRAND * 3.0 + 1) * (((random() % 1) * 2) - 1));
+  divisor = ((RAND() / MAXRAND * 3.0 + 1) * (((RAND() % 1) * 2) - 1));
 
-  if ((random() & 1) == 0)
+  if ((RAND() & 1) == 0)
     {
       radius1 = radius;
       radius2 = radius / divisor;
@@ -156,11 +156,11 @@ random_helix (win)
     }
 
   while (gcd (ANGLES, d_angle) >= 2)
-    d_angle = random () % ANGLES;
+    d_angle = RAND() % ANGLES;
 
 #define random_factor()				\
-  (((random() % 7) ? ((random() % 1) + 1) : 3)	\
-   * (((random() % 1) * 2) - 1))
+  (((RAND() % 7) ? ((RAND() % 1) + 1) : 3)	\
+   * (((RAND() % 1) * 2) - 1))
 
   while (gcd (gcd (gcd (factor1, factor2), factor3), factor4) != 1)
     {
