@@ -1,8 +1,8 @@
 #ifndef lint
-static char sccsid[] = "@(#)life3d.c	23.6 91/05/24 XLOCK";
+static char sccsid[] = "@(#)life3d.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
- * life3d.c - Extension to Conway's game of Life, Carter Bayes' 4555 3d life,
+ * life3d.c - Extension to Conway's game of Life, Carter Bays' 4555 3d life,
  *            for xlock, the X Window System lockscreen.
  *
  * Copyright (c) 1994 by David Bagley.
@@ -10,6 +10,7 @@ static char sccsid[] = "@(#)life3d.c	23.6 91/05/24 XLOCK";
  * See xlock.c for copying information.
  *
  * Revision History:
+ * 12-Feb-95: shooting gliders added
  * 07-Dec-94: used life.c and a DOS version of 3dlife
  * Copyright 1993 Anthony Wesley awesley@canb.auug.org.au found at
  * life.anu.edu.au /pub/complex_systems/alife/3DLIFE.ZIP
@@ -21,10 +22,10 @@ static char sccsid[] = "@(#)life3d.c	23.6 91/05/24 XLOCK";
  * Dewdney, A.K., "The Armchair Universe, Computer Recreations from the
  *   Pages of Scientific American Magazine", W.H. Freedman and Company,
  *   New York, 1988
- * Bayes, Carter, "The Game of Three Dimensional Life", 86/11/20
+ * Bays, Carter, "The Game of Three Dimensional Life", 86/11/20
+ *   with (latest?) update from 87/2/1
  */
 
-#include <malloc.h>
 #include <math.h>
 #include "xlock.h"
 
@@ -66,6 +67,8 @@ static char sccsid[] = "@(#)life3d.c	23.6 91/05/24 XLOCK";
 #define GREEN 2
 #define BLUE 3
 #define WHITE 4
+#define TIMEOUT 25
+#define NUMPTS  42
 
 typedef struct _CellList {
   unsigned char x, y, z;    /* Location in world coordinates */
@@ -77,10 +80,12 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
+  int initialized;
   int wireframe; /* FALSE */
   int ox, oy, oz; /* origin */
   double vx, vy, vz; /* viewpoint */
   int generation;
+  long shooterTime;
   int nstacks, nrows, ncolumns;
   int memstart;
   char visible;
@@ -97,30 +102,19 @@ typedef struct {
 
 static life3dstruct life3ds[MAXSCREENS];
 
-static int initialized = 0;
-
 static int DrawCube();
 
-static int patterns[][128] = {
-  {				/* GLIDER */
-    -1, -1, -1, 0, -1, -1,
-    -2, 0, -1, 1, 0, -1,
-    -2, 1, -1, 1, 1, -1,
-
-    -1, 0, 0, 0, 0, 0,
-    -1, 1, 0, 0, 1, 0,
-    127
-  },
-/* I hate comments in C */
+static int patterns[][3 * NUMPTS + 1] = {
+#if 0
 /* Still life */
-/*  {				*//* V *//*
+  {				/* V */
     -1, -1, -1, 0, -1, -1,
     -1, 0, -1, 0, 0, -1,
 
     -1, -1, 0, 0, -1, 0,
     127
   },
-  {				*//* CROSS *//*
+  {				/* CROSS */
     0, 0, -1,
 
     0, -1, 0,
@@ -129,7 +123,18 @@ static int patterns[][128] = {
 
     0, 0, 1,
     127
-  },*/
+  },
+  {				/* PILLAR */
+    0, -1, -1,
+    -1, 0, -1, 1, 0, -1,
+    0, 1, -1,
+
+    0, -1, 0,
+    -1, 0, 0, 1, 0, 0,
+    0, 1, 0,
+    127
+  },
+#endif
   {                             /* BLINKER */
     0, 0, -1,
 
@@ -586,11 +591,11 @@ void DisplayList()
 
   ptr=lp->ptrhead;
   while (ptr != NULL) {
-    printf("(%x)=[%d,%d,%d] ", (int)ptr, ptr->x, ptr->y, ptr->z);
+    (void) printf("(%x)=[%d,%d,%d] ", (int)ptr, ptr->x, ptr->y, ptr->z);
     ptr = ptr->next;
     ++count;
   }
-  printf("Living cells = %d\n", count);
+  (void) printf("Living cells = %d\n", count);
 }
 #endif
 
@@ -606,7 +611,7 @@ int n, v;
   for (z = lp->nstacks / 2 - v; z < lp->nstacks / 2 + v; ++z)
     for (y = lp->nrows / 2 - v; y < lp->nrows / 2 + v; ++y)
       for (x = lp->ncolumns / 2 - v; x < lp->ncolumns / 2 + v; ++x)
-        if ((random() % 100) < n)
+        if ((RAND() % 100) < n)
           SetList3D(x, y, z);
 }
 
@@ -756,7 +761,7 @@ int p1, p2, p3, p4;
     XSetForeground(dsp, Scr[screen].gc, lp->color[color]);
     XFillPolygon(dsp, win, Scr[screen].gc, facepts, 4, Convex, CoordModeOrigin);
   }
-  if (color == BLACK || ((mono || Scr[screen].npixels < 2) && !lp->wireframe))
+  if (color == BLACK || ((mono || Scr[screen].npixels <= 2) && !lp->wireframe))
     XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
   else
     XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
@@ -864,8 +869,12 @@ Window win;
   int i;
 
   lp->generation = 0;
+  lp->shooterTime = seconds();
 
-  if (!initialized) {
+  if (!lp->initialized) {
+    /* 4555 life
+       (5766 life is good too (has lifeforms and a glider similar to life)
+       also 5655 and 6767 are interesting) */
     lp->living_min = 4;
     lp->living_max = 5;
     lp->newcell_min = 5;
@@ -880,20 +889,20 @@ Window win;
     lp->oy = lp->nrows / 2;
     lp->oz = lp->nstacks / 2;
     lp->wireframe = 0;
-    initialized = 1;
+    lp->initialized = 1;
     Init3D();
   } else
     End3D();
   (void) XGetWindowAttributes(dsp, win, &xgwa);
   lp->color[0] = BlackPixel(dsp, screen);
   if (!mono && Scr[screen].npixels > 2) {
-    i = random() % 3;
+    i = RAND() % 3;
 
-    lp->color[i + 1] = Scr[screen].pixels[random() %
+    lp->color[i + 1] = Scr[screen].pixels[RAND() %
       (Scr[screen].npixels / COLORBASE)];
-    lp->color[(i + 1) % 3 + 1] = Scr[screen].pixels[random() %
+    lp->color[(i + 1) % 3 + 1] = Scr[screen].pixels[RAND() %
       (Scr[screen].npixels / COLORBASE) + Scr[screen].npixels / COLORBASE];
-    lp->color[(i + 2) % 3 + 1] = Scr[screen].pixels[random() %
+    lp->color[(i + 2) % 3 + 1] = Scr[screen].pixels[RAND() %
       (Scr[screen].npixels / COLORBASE) + 2 * Scr[screen].npixels / COLORBASE];
   } else
     lp->color[1] = lp->color[2] = lp->color[3] = WhitePixel(dsp, screen);
@@ -916,7 +925,7 @@ Window win;
   lp->vz = (sin(lp->alt * IP) * lp->dist);
   NewViewpoint(lp->vx, lp->vy, lp->vz);
 
-  i = random() % (NPATS + 2);
+  i = RAND() % (NPATS + 2);
   if (i >= NPATS)
     RandomSoup(30, 10);
   else
@@ -925,6 +934,82 @@ Window win;
   DrawScreen(win);
 }
 
+static void shooter()
+{
+    int hsp, vsp, asp, hoff = 1, voff = 1, aoff = 1, r, c2, r2, s2;
+    life3dstruct *lp = &life3ds[screen];
+
+    /* Generate the glider at the edge of the screen */
+#define V 10
+#define V2 (V/2)
+    c2 = lp->ncolumns / 2;
+    r2 = lp->nrows / 2;
+    s2 = lp->nstacks / 2;
+    r = RAND() % 3;
+    if (!r) {
+      hsp = RAND() % V2 + c2  - V2 / 2;
+      vsp = (RAND() % 2) ? r2 - V : r2 + V;
+      asp = (RAND() % 2) ? s2 - V : s2 + V;
+      if (asp > s2)
+        aoff = -1;
+      if (vsp > r2)
+        voff = -1;
+      if (hsp > c2)
+        hoff = -1;
+      SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 2 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 3 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 3 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+    } else if (r == 1) {
+      hsp = (RAND() % 2) ? c2 - V : c2 + V;
+      vsp = (RAND() % 2) ? r2 - V : r2 + V;
+      asp = RAND() % V2 + s2 - V2 / 2;
+      if (asp > s2)
+        aoff = -1;
+      if (vsp > r2)
+        voff = -1;
+      if (hsp > c2)
+        hoff = -1;
+      SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 2 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 3 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 3 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+    } else {
+      hsp = (RAND() % 2) ? c2 - V : c2 + V;
+      vsp = RAND() % V2 + r2 - V2 / 2;
+      asp = (RAND() % 2) ? s2 - V : s2 + V;
+      if (asp > s2)
+        aoff = -1;
+      if (vsp > r2)
+        voff = -1;
+      if (hsp > c2)
+        hoff = -1;
+      SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 2 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 0 * hoff, vsp + 3 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
+      SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
+    }
+    lp->shooterTime = seconds();
+}
+
 void drawlife3d(win)
 Window win;
 {
@@ -933,8 +1018,15 @@ Window win;
   RunLife3D();
   DrawScreen(win);
 
-  if (++lp->generation > batchcount ||
-     !lp->visible)
+  if (++lp->generation > batchcount || !lp->visible)
      /*CountCells3D() == 0)*/
     initlife3d(win);
+
+    /*
+     * generate a randomized shooter aimed roughly toward the center of the
+     * screen after timeout.
+     */
+
+    if (seconds() - lp->shooterTime > TIMEOUT)  
+        shooter();
 }
