@@ -1,5 +1,5 @@
 #ifndef lint
-static  char sccsid[] = "@(#)maze.c 1.4 86/12/29 XLOCK";
+static  char sccsid[] = "@(#)maze.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * maze.c - A maze for xlock, the X Window System lockscreen.
@@ -44,7 +44,7 @@ static  char sccsid[] = "@(#)maze.c 1.4 86/12/29 XLOCK";
 #include <stdio.h>
 #include "xlock.h"
 #include <X11/bitmaps/gray1>
-#include "mazeicon.bit"
+#include "maze.xbm"
 
 #define MIN_MAZE_SIZE	3
 #define MAX_SIZE        40
@@ -75,7 +75,7 @@ static  char sccsid[] = "@(#)maze.c 1.4 86/12/29 XLOCK";
 #define START_SQUARE	0x2
 #define END_SQUARE	0x1
 
-#define get_random(x)	(random() % (x))
+#define get_random(x)	(RAND() % (x))
 
 static XImage logo = {
     0, 0,                       /* width, height */
@@ -101,6 +101,7 @@ typedef struct {
         u_short maze[MAX_MAZE_SIZE_X][MAX_MAZE_SIZE_Y];
         paths move_list[MOVE_LIST_SIZE];
         paths save_path[MOVE_LIST_SIZE], path[MOVE_LIST_SIZE];
+        int solving;
         } mazestruct;
 
 static mazestruct mazes[MAXSCREENS];
@@ -119,12 +120,12 @@ initmaze(win)
     Pixmap gray;
     mazestruct *mp = &mazes[screen];
 
-    logo.data = (char *) mazeicon_bits;
-    logo.width = mazeicon_width;
-    logo.height = mazeicon_height;
-    logo.bytes_per_line = (mazeicon_width + 7) / 8;
+    logo.data = (char *) maze_bits;
+    logo.width = maze_width;
+    logo.height = maze_height;
+    logo.bytes_per_line = (maze_width + 7) / 8;
     
-    XGetWindowAttributes(dsp, win, &xgwa);
+    (void) XGetWindowAttributes(dsp, win, &xgwa);
     mp->width = xgwa.width;
     mp->height = xgwa.height;
     mp->gray_GC = XCreateGC(dsp, win, 0, 0);
@@ -134,6 +135,7 @@ initmaze(win)
     XSetStipple(dsp, mp->gray_GC, gray);
     XSetFillStyle(dsp, mp->gray_GC, FillOpaqueStippled);
     mp->counter = 0;
+    mp->solving = 0;
 }
 
 
@@ -192,7 +194,7 @@ initialize_maze() /* draw the surrounding wall and start/end squares */
         XSetForeground(dsp, mp->gray_GC, WhitePixel(dsp, screen));
     }
     if (Scr[screen].npixels > 2) {
-        i = random() % Scr[screen].npixels;
+        i = RAND() % Scr[screen].npixels;
         XSetForeground(dsp, Scr[screen].gc, Scr[screen].pixels[i]);
         XSetForeground(dsp, mp->gray_GC, Scr[screen].pixels[i]);
     }
@@ -472,7 +474,7 @@ draw_maze_border(win) /* draw the maze outline */
 			mp->border_y + mp->sq_size_y * mp->logo_y +
                           (mp->sq_size_y * mp->logo_size_y - logo.height + 1) /
 			  2,
-			mazeicon_width, mazeicon_height);
+			maze_width, maze_height);
 	}
 
 	draw_solid_square( win, mp->start_x, mp->start_y, mp->start_dir,
@@ -559,22 +561,26 @@ solve_maze(win) /* solve it with graphical feedback */
         Window win;
 {
         mazestruct *mp = &mazes[screen];
-	int i;
+	static int i;
+	int n;
 
+	if (!mp->solving) {
+	    /* plug up the surrounding wall */
+	    mp->maze[mp->start_x][mp->start_y] |= (WALL_TOP >> mp->start_dir);
+	    mp->maze[mp->end_x][mp->end_y] |= (WALL_TOP >> mp->end_dir);
 
-	/* plug up the surrounding wall */
-	mp->maze[mp->start_x][mp->start_y] |= (WALL_TOP >> mp->start_dir);
-	mp->maze[mp->end_x][mp->end_y] |= (WALL_TOP >> mp->end_dir);
+	    /* initialize search path */
+	    i = 0;
+	    mp->path[i].x = mp->end_x;
+	    mp->path[i].y = mp->end_y;
+	    mp->path[i].dir = -1;
 
-	/* initialize search path */
-	i = 0;
-	mp->path[i].x = mp->end_x;
-	mp->path[i].y = mp->end_y;
-	mp->path[i].dir = -1;
+	    mp->solving = 1;
+	}
 
-	/* do it */
-	while (1) {
- 	    if ( ++mp->path[i].dir >= 4 ) {
+	/* do it (5 squares at a time) */
+	for(n = 0; n < 5; n++) {
+	    if ( ++mp->path[i].dir >= 4 ) {
 		i--;
 		draw_solid_square( win, 
 			(int)(mp->path[i].x), (int)(mp->path[i].y),
@@ -587,6 +593,7 @@ solve_maze(win) /* solve it with graphical feedback */
 		enter_square(win, i);
 		i++;
 		if ( mp->maze[mp->path[i].x][mp->path[i].y] & START_SQUARE ) {
+			mp->solving = 0;
 			return;
 		}
 	    }
@@ -627,6 +634,11 @@ drawmaze(win)
     Window win;
 {
     mazestruct *mp = &mazes[screen];
+
+    if(mp->solving) {
+      solve_maze(win);
+      return;
+    }
     switch (mp->counter)
     {
       case 0:
@@ -641,20 +653,19 @@ drawmaze(win)
 	create_maze(win);
         break;
       case 3:
-      case 4:
 	(void) sleep(1);
         break;
-      case 5:
+      case 4:
 	solve_maze(win);
         break;
+      case 5:
       case 6:
       case 7:
       case 8:
-      case 9:
       	(void) sleep(1);
         break;
     }
-    if (mp->counter == 9)
+    if (mp->counter == 8)
       mp->counter = 0;
     else
       mp->counter++;
