@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)pyro.c	1.1 91/05/24 XLOCK";
+static char sccsid[] = "@(#)pyro.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * pyro.c - Fireworks for xlock, the X Window System lockscreen.
@@ -62,8 +62,8 @@ static char sccsid[] = "@(#)pyro.c	1.1 91/05/24 XLOCK";
 #define MINSFUSE 50		/* Range of fuse lengths for stars */
 #define MAXSFUSE 100
 
-#define INTRAND(min,max) (random()%((max+1)-(min))+(min))
-#define FLOATRAND(min,max) ((min)+((double) random()/((double) MAXRAND))*((max)-(min)))
+#define INTRAND(min,max) (RAND()%((max+1)-(min))+(min))
+#define FLOATRAND(min,max) ((min)+((double) RAND()/((double) MAXRAND))*((max)-(min)))
 
 static void ignite();
 static void animate();
@@ -188,9 +188,9 @@ drawpyro(win)
     rocket     *rp;
     int         rockn;
 
-    if (just_started || (random() % pp->p_ignite == 0)) {
+    if (just_started || (RAND() % pp->p_ignite == 0)) {
 	just_started = False;
-	if (random() % P_FUSILLADE == 0) {
+	if (RAND() % P_FUSILLADE == 0) {
 	    pp->p_ignite = orig_p_ignite / FUSILFACTOR;
 	    pp->fusilcount = INTRAND(FUSILLEN * 9 / 10, FUSILLEN * 11 / 10);
 	}
@@ -217,7 +217,7 @@ ignite(pp)
     unsigned long	color1, color2;
     float       xvel, yvel, x;
 
-    x = random() % pp->width;
+    x = RAND() % pp->width;
     xvel = FLOATRAND(-pp->maxvelx, pp->maxvelx);
 /* All this to stop too many rockets going offscreen: */
     if ((x < pp->lmargin && xvel < 0.0) || (x > pp->rmargin && xvel > 0.0))
@@ -226,18 +226,18 @@ ignite(pp)
     fuse = INTRAND(MINFUSE, MAXFUSE);
     nstars = INTRAND(MINSTARS, MAXSTARS);
     if (!mono && (npix = Scr[screen].npixels) > 2) {
-	color1 = Scr[screen].pixels[pix = random() % npix];
+	color1 = Scr[screen].pixels[pix = RAND() % npix];
 	color2 = Scr[screen].pixels[(pix + (npix / 2)) % npix];
     } else {
 	color1 = color2 = WhitePixel(dsp, screen);
     }
 
     multi = 1;
-    if (random() % P_DOUBLECLOUD == 0)
+    if (RAND() % P_DOUBLECLOUD == 0)
 	shelltype = DOUBLECLOUD;
     else {
 	shelltype = CLOUD;
-	if (random() % P_MULTI == 0)
+	if (RAND() % P_MULTI == 0)
 	    multi = INTRAND(5, 15);
     }
 
@@ -316,7 +316,7 @@ shootup(win, pp, rp)
     rp->yvel += pp->rockdecel;
     XSetForeground(dsp, Scr[screen].gc, pp->rockpixel);
     XFillRectangle(dsp, win, Scr[screen].gc, (int) (rp->x), (int) (rp->y),
-		   ROCKETW, ROCKETH + random() % 4);
+		   ROCKETW, ROCKETH + RAND() % 4);
 }
 
 static void
