@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)qix.c	23.8 91/05/24 XLOCK";
+static char sccsid[] = "@(#)qix.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * qix.c - Vector swirl for xlock, the X Window System lockscreen.
@@ -74,14 +74,14 @@ initqix(win)
     qp->offset = qp->delta / 3;
     qp->last = 0;
     qp->pix = 0;
-    qp->dx1 = random() % qp->delta + qp->offset;
-    qp->dy1 = random() % qp->delta + qp->offset;
-    qp->dx2 = random() % qp->delta + qp->offset;
-    qp->dy2 = random() % qp->delta + qp->offset;
-    qp->x1 = random() % qp->width;
-    qp->y1 = random() % qp->height;
-    qp->x2 = random() % qp->width;
-    qp->y2 = random() % qp->height;
+    qp->dx1 = RAND() % qp->delta + qp->offset;
+    qp->dy1 = RAND() % qp->delta + qp->offset;
+    qp->dx2 = RAND() % qp->delta + qp->offset;
+    qp->dy2 = RAND() % qp->delta + qp->offset;
+    qp->x1 = RAND() % qp->width;
+    qp->y1 = RAND() % qp->height;
+    qp->x2 = RAND() % qp->width;
+    qp->y2 = RAND() % qp->height;
     XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
     XFillRectangle(dsp, win, Scr[screen].gc, 0, 0, qp->width, qp->height);
 }
@@ -89,9 +89,9 @@ initqix(win)
 #define check_bounds(qp, val, del, max)				\
 {								\
     if ((val) < 0) {						\
-	*(del) = (random() % (qp)->delta) + (qp)->offset;	\
+	*(del) = (RAND() % (qp)->delta) + (qp)->offset;	\
     } else if ((val) > (max)) {					\
-	*(del) = -(random() % (qp)->delta) - (qp)->offset;	\
+	*(del) = -(RAND() % (qp)->delta) - (qp)->offset;	\
     }								\
 }
 
