@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)resource.c	1.20 91/09/27 XLOCK";
+static char sccsid[] = "@(#)resource.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * resource.c - resource management for xlock, the X Window System lockscreen.
@@ -137,9 +137,7 @@ extern void drawbomb();
 extern void initblank();
 extern void drawblank();
 
-#ifdef FORCE_BOMB
 extern int full_lock();
-#endif
 
 typedef struct {
     char       *cmdline_arg;
@@ -152,7 +150,6 @@ typedef struct {
 }           LockStruct;
 
 #define NUMSPECIAL 4 /* # of Special Screens */
-static char swirlstring[] = "swirl";
 static char randomstring[] = "random";
 static char bombstring[] = "bomb";
 
@@ -180,8 +177,8 @@ static LockStruct LockProcs[] = {
     {"galaxy", initgalaxy, drawgalaxy, 100, 3, 1.0, "Spinning galaxies"},
     {"kaleid", initkaleid, drawkaleid, 2000, 4, 1.0, "Kaleidoscope"},
     {"wator", initwator, drawwator, 750000, 4, 1.0, "Dewdney's Planet Wa-Tor"},
-    {"life3d", initlife3d, drawlife3d, 1000000, 40, 1.0, "Bayes' 3D Life"},
-    {swirlstring, initswirl, drawswirl, 5000, 5, 1.0, "Animated swirling patterns"},
+    {"life3d", initlife3d, drawlife3d, 1000000, 80, 1.0, "Bays' 3D Life"},
+    {"swirl", initswirl, drawswirl, 5000, 5, 1.0, "Animated swirling patterns"},
     {"image", initimage, drawimage, 2000000, 1, 1.0, "Random Bouncing Image"},
     {bombstring, initbomb, drawbomb, 1000000, 1, 1.0, "Temporary screen lock"},
     {"blank", initblank, drawblank, 4000000, 1, 1.0, "Blank screen"},
@@ -206,7 +203,7 @@ extern char *getenv();
 #ifdef VMS
 #define DEF_DISPLAY     "DECW$DISPLAY:"
 #else
-#define DEF_DISPLAY	":0"
+#define DEF_DISPLAY	""
 #endif
 #define DEF_MODE	"blank"
 #ifndef AIXV3
@@ -227,6 +224,24 @@ extern char *getenv();
 #define DEF_NICE	"10"	/* xlock process nicelevel */
 #define DEF_SAT		"1.0"	/* color ramp saturation 0->1 */
 #define DEF_CLASSNAME	"XLock"
+#define DEF_GEOMETRY	""
+
+#ifdef AUTO_LOGOUT
+#define DEF_LOGOUT	"30"    /* minutes until auto-logout */
+#endif
+
+#ifdef LOGOUT_BUTTON
+
+#define DEF_BTN_LABEL	"Logout" /* string that appears in logout button */
+
+/* this string appears immediately below logout button */
+#define DEF_BTN_HELP	"Click here to logout"
+
+/* this string appears in place of the logout button if user could not
+   be logged out */
+#define DEF_FAIL	"Auto-logout failed"
+
+#endif
 
 static char *classname;
 static char modename[1024];
@@ -240,12 +255,16 @@ static XrmOptionDescRec genTable[] = {
     {"+remote", ".remote", XrmoptionNoArg, (caddr_t) "off"},
     {"-mono", ".mono", XrmoptionNoArg, (caddr_t) "on"},
     {"+mono", ".mono", XrmoptionNoArg, (caddr_t) "off"},
+#ifndef ALWAYS_ALLOW_ROOT
     {"-allowroot", ".allowroot", XrmoptionNoArg, (caddr_t) "on"},
     {"+allowroot", ".allowroot", XrmoptionNoArg, (caddr_t) "off"},
+#endif
     {"-enablesaver", ".enablesaver", XrmoptionNoArg, (caddr_t) "on"},
     {"+enablesaver", ".enablesaver", XrmoptionNoArg, (caddr_t) "off"},
     {"-allowaccess", ".allowaccess", XrmoptionNoArg, (caddr_t) "on"},
     {"+allowaccess", ".allowaccess", XrmoptionNoArg, (caddr_t) "off"},
+    {"-grabmouse", ".grabmouse", XrmoptionNoArg, (caddr_t) "on"},
+    {"+grabmouse", ".grabmouse", XrmoptionNoArg, (caddr_t) "off"},
     {"-echokeys", ".echokeys", XrmoptionNoArg, (caddr_t) "on"},
     {"+echokeys", ".echokeys", XrmoptionNoArg, (caddr_t) "off"},
     {"-usefirst", ".usefirst", XrmoptionNoArg, (caddr_t) "on"},
@@ -258,6 +277,8 @@ static XrmOptionDescRec genTable[] = {
     {"+inroot",".inroot", XrmoptionNoArg, (caddr_t) "off"},
     {"-timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "on"},
     {"+timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "off"},
+    {"-install", ".install", XrmoptionNoArg, (caddr_t) "on"},
+    {"+install", ".install", XrmoptionNoArg, (caddr_t) "off"},
     {"-nice", ".nice", XrmoptionSepArg, (caddr_t) NULL},
     {"-timeout", ".timeout", XrmoptionSepArg, (caddr_t) NULL},
     {"-font", ".font", XrmoptionSepArg, (caddr_t) NULL},
@@ -270,6 +291,16 @@ static XrmOptionDescRec genTable[] = {
     {"-info", ".info", XrmoptionSepArg, (caddr_t) NULL},
     {"-validate", ".validate", XrmoptionSepArg, (caddr_t) NULL},
     {"-invalid", ".invalid", XrmoptionSepArg, (caddr_t) NULL},
+    {"-geometry", ".geometry", XrmoptionSepArg, (caddr_t) NULL},
+#ifdef AUTO_LOGOUT
+    {"-forceLogout", ".forceLogout", XrmoptionSepArg, (caddr_t) NULL},
+#endif
+#ifdef LOGOUT_BUTTON
+    {"-logoutButtonLabel", ".logoutButtonLabel", XrmoptionSepArg, (caddr_t) NULL},
+    {"-logoutButtonHelp", ".logoutButtonHelp", XrmoptionSepArg, (caddr_t) NULL},
+    {"-logoutFailedString", ".logoutFailedString", XrmoptionSepArg, (caddr_t) NULL},
+#endif
+
 };
 #define genEntries (sizeof genTable / sizeof genTable[0])
 
@@ -312,15 +343,19 @@ static OptionStruct opDesc[] = {
     {"-/+mono", "turn on/off monochrome override"},
     {"-/+nolock", "turn on/off no password required mode"},
     {"-/+remote", "turn on/off remote host access"},
+#ifndef ALWAYS_ALLOW_ROOT
     {"-/+allowroot", "turn on/off allow root password mode"},
+#endif
     {"-/+enablesaver", "turn on/off enable X server screen saver"},
     {"-/+allowaccess", "turn on/off allow new clients to connect"},
+    {"-/+grabmouse", "turn on/off grabbing of mouse and keyboard"},
     {"-/+echokeys", "turn on/off echo '?' for each password key"},
     {"-/+usefirst", "turn on/off using the first char typed in password"},
     {"-/+v", "turn on/off verbose mode"},
     {"-/+inwindow", "turn on/off making xlock run in a window"},
     {"-/+inroot", "turn on/off making xlock run in the root window"},
     {"-/+timeelapsed", "turn on/off clock"},
+    {"-/+install", "whether to use private colormap if needed (yes/no)"},
     {"-delay usecs", "microsecond delay between screen updates"},
     {"-batchcount num", "number of things per batch"},
     {"-nice level", "nice level for xlock process"},
@@ -334,6 +369,7 @@ static OptionStruct opDesc[] = {
     {"-info string", "text string to use for instructions"},
     {"-validate string", "text string to use for validating password message"},
     {"-invalid string", "text string to use for invalid password message"},
+    {"-geometry geom", "geometry for non-full screen lock"},
 };
 #define opDescEntries (sizeof opDesc / sizeof opDesc[0])
 
@@ -347,23 +383,39 @@ char       *text_pass;
 char       *text_info;
 char       *text_valid;
 char       *text_invalid;
+char       *geometry;
 float       saturation;
 int         nicelevel;
 int         delay;
 int         batchcount;
 int         timeout;
+#ifdef AUTO_LOGOUT
+int         forceLogout;
+#endif
+#ifdef LOGOUT_BUTTON
+int         enable_button = 1;
+char       *logoutButtonLabel;
+char       *logoutButtonHelp;
+char       *logoutFailedString;
+#endif
 Bool        mono;
 Bool        nolock;
 Bool        remote;
+#ifdef ALWAYS_ALLOW_ROOT
+Bool        allowroot = 1;
+#else
 Bool        allowroot;
+#endif
 Bool        enablesaver;
 Bool        allowaccess;
+Bool        grabmouse;
 Bool        echokeys;
 Bool        usefirst;
 Bool        verbose;
 Bool        inwindow;
 Bool        inroot;
 Bool        timeelapsed;
+Bool        install;
 
 #define t_String	0
 #define t_Float		1
@@ -387,20 +439,37 @@ static argtype genvars[] = {
     {(caddr_t *) &text_info, "info", "Info", DEF_INFO, t_String},
     {(caddr_t *) &text_valid, "validate", "Validate", DEF_VALID, t_String},
     {(caddr_t *) &text_invalid, "invalid", "Invalid", DEF_INVALID, t_String},
+    {(caddr_t *) &geometry, "geometry", "Geometry", DEF_GEOMETRY, t_String},
     {(caddr_t *) &nicelevel, "nice", "Nice", DEF_NICE, t_Int},
     {(caddr_t *) &timeout, "timeout", "Timeout", DEF_TIMEOUT, t_Int},
     {(caddr_t *) &mono, "mono", "Mono", "off", t_Bool},
     {(caddr_t *) &nolock, "nolock", "NoLock", "off", t_Bool},
     {(caddr_t *) &remote, "remote", "Remote", "off", t_Bool},
+#ifndef ALWAYS_ALLOW_ROOT
     {(caddr_t *) &allowroot, "allowroot", "AllowRoot", "off", t_Bool},
+#endif
     {(caddr_t *) &enablesaver, "enablesaver", "EnableSaver", "off", t_Bool},
     {(caddr_t *) &allowaccess, "allowaccess", "AllowAccess", "off", t_Bool},
+    {(caddr_t *) &grabmouse, "grabmouse", "GrabMouse", "on", t_Bool},
     {(caddr_t *) &echokeys, "echokeys", "EchoKeys", "off", t_Bool},
     {(caddr_t *) &usefirst, "usefirst", "Usefirst", "off", t_Bool},
     {(caddr_t *) &verbose, "verbose", "Verbose", "off", t_Bool},
     {(caddr_t *) &inwindow, "inwindow", "InWindow", "off", t_Bool},
     {(caddr_t *) &inroot, "inroot", "InRoot", "off", t_Bool},
     {(caddr_t *) &timeelapsed, "timeelapsed", "TimeElapsed", "off", t_Bool},
+    {(caddr_t *) &install, "install", "Install", "off", t_Bool},
+#ifdef AUTO_LOGOUT
+    {(caddr_t *) &forceLogout, "forceLogout", "ForceLogout", DEF_LOGOUT, t_Int
+},
+#endif
+#ifdef LOGOUT_BUTTON
+    {(caddr_t *) &logoutButtonLabel, "logoutButtonLabel",
+               "LogoutButtonLabel", DEF_BTN_LABEL, t_String},
+    {(caddr_t *) &logoutButtonHelp, "logoutButtonHelp",
+               "LogoutButtonHelp", DEF_BTN_HELP, t_String},
+    {(caddr_t *) &logoutFailedString, "logoutFailedString",
+               "LogoutFailedString", DEF_FAIL, t_String},
+#endif
 };
 #define NGENARGS (sizeof genvars / sizeof genvars[0])
 
@@ -839,19 +908,14 @@ GetResources(argc, argv)
 		DEF_MODE, (caddr_t *) &mode);
 
     /*
-     * if random< mode, then just grab a random entry from the table
-     * or root is trying to run bomb ... sorry not a good idea,
-     * attempting to run swirl in root window is not a good idea either
+     * if RAND < mode, then just grab a random entry from the table
+     * or root is trying to run bomb ... sorry not a good idea
      */
    
     if (!strcmp(mode, randomstring))
-      mode = LockProcs[random() % (NUMPROCS - NUMSPECIAL)].cmdline_arg;
+      mode = LockProcs[RAND() % (NUMPROCS - NUMSPECIAL)].cmdline_arg;
     if (!getuid() && !strcmp(mode, bombstring))
        error("%s: mode %s can not run as root.\n", bombstring);
-#ifndef FORCEFIXEDCOLORS
-    if (!mono && inroot && !strcmp(mode, swirlstring))
-       error("%s: mode %s can not run in root window.\n", swirlstring);
-#endif
 #ifdef FORCE_BOMB
     /*
      *  Force temporary locking mode
@@ -899,6 +963,17 @@ GetResources(argc, argv)
 
     (void) XrmDestroyDatabase(RDB);
 
+#ifndef FORCE_BOMB
+     if ( full_lock() ) {
+#ifdef AUTO_LOGOUT
+       forceLogout = 0;
+#endif
+#ifdef LOGOUT_BUTTON
+       enable_button = 0;
+#endif
+     }
+#endif
+
     if (verbose) {
 	for (i = 0; i < NGENARGS; i++)
 	    printvar(classname, genvars[i]);
