@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)sphere.c 1.3 88/02/26 XLOCK";
+static char sccsid[] = "@(#)sphere.c	2.7 95/02/21 xlockmore";
 #endif
 
 /*-
@@ -91,13 +91,13 @@ drawsphere(win)
     register      y;
 
     if (ss->x >= ss->radius) {
-	ss->radius = random() % (MIN(ss->width / 2, ss->height / 2) - 1) + 1;
-	ss->x0 = random() % ss->width;
-	ss->y0 = random() % ss->height;
+	ss->radius = RAND() % (MIN(ss->width / 2, ss->height / 2) - 1) + 1;
+	ss->x0 = RAND() % ss->width;
+	ss->y0 = RAND() % ss->height;
 	ss->x = -ss->radius;
 
         if (Scr[screen].npixels > 2)
-            ss->color = random() % Scr[screen].npixels;
+            ss->color = RAND() % Scr[screen].npixels;
     }
     ss->maxy = SQRT(ss->radius * ss->radius - ss->x * ss->x);
     XSetForeground(dsp, Scr[screen].gc, BlackPixel(dsp, screen));
@@ -109,7 +109,7 @@ drawsphere(win)
     else
         XSetForeground(dsp, Scr[screen].gc, WhitePixel(dsp, screen));
     for (y = -ss->maxy; y <= ss->maxy; y++)
-        if ((random() % (ss->radius * NR)) <=
+        if ((RAND() % (ss->radius * NR)) <=
                 (NX * ss->x + NY * y + NZ *
 		 SQRT(ss->radius * ss->radius - ss->x * ss->x - y * y)))
             XDrawPoint(dsp, win, Scr[screen].gc,
