@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)swarm.c	1.5 91/05/24 XLOCK";
+static char sccsid[] = "@(#)swarm.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * swarm.c - swarm of bees for xlock, the X Window System lockscreen.
@@ -13,15 +13,15 @@ static char sccsid[] = "@(#)swarm.c	1.5 91/05/24 XLOCK";
 #include "xlock.h"
 
 #define TIMES	4		/* number of time positions recorded */
-#define BEEACC	3		/* acceleration of bees */
+#define BEEACC	2		/* acceleration of bees */
 #define WASPACC 5		/* maximum acceleration of wasp */
-#define BEEVEL	11		/* maximum bee velocity */
-#define WASPVEL 12		/* maximum wasp velocity */
+#define BEEVEL	12		/* maximum bee velocity */
+#define WASPVEL 10		/* maximum wasp velocity */
 
 /* Macros */
 #define X(t,b)	(sp->x[(t)*sp->beecount+(b)])
 #define Y(t,b)	(sp->y[(t)*sp->beecount+(b)])
-#define RAND(v)	((random()%(v))-((v)/2))	/* random number around 0 */
+#define balance_rand(v)	((RAND()%(v))-((v)/2))	/* random number around 0 */
 
 typedef struct {
     int         pix;
@@ -53,7 +53,7 @@ initswarm(win)
 
     sp->beecount = batchcount;
 
-    XGetWindowAttributes(dsp, win, &xgwa);
+    (void) XGetWindowAttributes(dsp, win, &xgwa);
     sp->width = xgwa.width;
     sp->height = xgwa.height;
     sp->border = (sp->width + sp->height) / 50;
@@ -75,8 +75,8 @@ initswarm(win)
     /* Initialize point positions, velocities, etc. */
 
     /* wasp */
-    sp->wx[0] = sp->border + random() % (sp->width - 2 * sp->border);
-    sp->wy[0] = sp->border + random() % (sp->height - 2 * sp->border);
+    sp->wx[0] = sp->border + RAND() % (sp->width - 2 * sp->border);
+    sp->wy[0] = sp->border + RAND() % (sp->height - 2 * sp->border);
     sp->wx[1] = sp->wx[0];
     sp->wy[1] = sp->wy[0];
     sp->wxv = 0;
@@ -84,12 +84,12 @@ initswarm(win)
 
     /* bees */
     for (b = 0; b < sp->beecount; b++) {
-	X(0, b) = random() % sp->width;
+	X(0, b) = RAND() % sp->width;
 	X(1, b) = X(0, b);
-	Y(0, b) = random() % sp->height;
+	Y(0, b) = RAND() % sp->height;
 	Y(1, b) = Y(0, b);
-	sp->xv[b] = RAND(7);
-	sp->yv[b] = RAND(7);
+	sp->xv[b] = balance_rand(7);
+	sp->yv[b] = balance_rand(7);
     }
 }
 
@@ -109,8 +109,8 @@ drawswarm(win)
     sp->wy[2] = sp->wy[1];
     sp->wy[1] = sp->wy[0];
     /* Accelerate */
-    sp->wxv += RAND(WASPACC);
-    sp->wyv += RAND(WASPACC);
+    sp->wxv += balance_rand(WASPACC);
+    sp->wyv += balance_rand(WASPACC);
 
     /* Speed Limit Checks */
     if (sp->wxv > WASPVEL)
@@ -136,8 +136,8 @@ drawswarm(win)
 	sp->wy[0] += sp->wyv;
     }
     /* Don't let things settle down. */
-    sp->xv[random() % sp->beecount] += RAND(3);
-    sp->yv[random() % sp->beecount] += RAND(3);
+    sp->xv[RAND() % sp->beecount] += balance_rand(3);
+    sp->yv[RAND() % sp->beecount] += balance_rand(3);
 
     /* <=- Bees -=> */
     for (b = 0; b < sp->beecount; b++) {
