@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)swirl.c     23.6 94/12/13 XLOCK";
+static char sccsid[] = "@(#)swirl.c	2.7 95/02/21 xlockmore";
 #endif
 /*
  * swirl.c - swirly patterns for xlock
@@ -131,7 +131,7 @@ static  SWIRL_P swirls = NULL ;
 static  int     random_no(n)
 unsigned int n;
 {
-  return((int)((n+1) * (double)random() / MAXRAND)) ;
+  return((int)((n+1) * (double)RAND() / MAXRAND)) ;
 }
 
 /****************************************************************/
@@ -716,7 +716,7 @@ SWIRL_P swirl;
   swirl->fixed_colourmap = ((default_info->class == StaticGray) ||
           (default_info->class == StaticColor) ||
           (default_info->class == TrueColor) ||
-          swirl->monochrome) ;
+          swirl->monochrome || inroot) ;
 
 #ifdef FORCEFIXEDCOLORS
   /* pretending a fixed colourmap */
@@ -724,7 +724,7 @@ SWIRL_P swirl;
 #endif
 
   /* free the list */
-  XFree(list) ;
+  XFree((char *)list) ;
 
   /* if monochrome, we've done all we can */
   if (swirl->monochrome)
@@ -963,11 +963,14 @@ Window win;
   create_colourmap(swirl) ;
 
   /* attach the colour map to the window (if we have one) */
-  if (!swirl->fixed_colourmap)
-  {
+  if (!swirl->fixed_colourmap) {
+#if 1
+    set_colormap(screen, swirl->cmap);
+#else
     XSetWindowColormap(dsp, win, swirl->cmap) ;
     (void) XSetWMColormapWindows(dsp, win, &win, 1) ;
     XInstallColormap(dsp, swirl->cmap) ;
+#endif
   }
 
   /* resolution starts off chunky */
