@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)worm.c 1.3 91/01/01 XLOCK";
+static char sccsid[] = "@(#)worm.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * worm.c - draw wiggly worms.
@@ -90,7 +90,7 @@ worm_doit(win, wp, which, color)
     y = ws->ycirc[ws->tail];
     XClearArea(dsp, win, x, y, CIRCSIZE, CIRCSIZE, False);
 
-    if (random() & 1) {
+    if (RAND() & 1) {
 	ws->dir = (ws->dir + 1) % SEGMENTS;
     } else {
 	ws->dir = (ws->dir + SEGMENTS - 1) % SEGMENTS;
@@ -137,13 +137,11 @@ initworm(win)
     wp->xsize = xwa.width;
     wp->ysize = xwa.height;
 
-    if (xwa.width < 100) {
-	wp->monopix = BlackPixel(dsp, screen);
+    wp->monopix = WhitePixel(dsp, screen);
+    if (xwa.width < 100)
 	wp->wormlength = MAXWORMLEN / 10;
-    } else {
-	wp->monopix = WhitePixel(dsp, screen);
+    else
 	wp->wormlength = MAXWORMLEN;
-    }
 
     for (i = 0; i < wp->nc; i++) {
 	for (j = 0; j < wp->nw / wp->nc + 1; j++) {
@@ -158,7 +156,7 @@ initworm(win)
 	    wp->worm[i].xcirc[j] = wp->xsize / 2;
 	    wp->worm[i].ycirc[j] = wp->ysize / 2;
 	}
-	wp->worm[i].dir = (unsigned) random() % SEGMENTS;
+	wp->worm[i].dir = (unsigned) RAND() % SEGMENTS;
 	wp->worm[i].tail = 0;
 	wp->worm[i].x = wp->xsize / 2;
 	wp->worm[i].y = wp->ysize / 2;
