@@ -1,5 +1,5 @@
 #ifndef lint
-static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
+static char sccsid[] = "@(#)xlock.c	2.7 95/02/21 xlockmore";
 #endif
 /*-
  * xlock.c - X11 client to lock a display and show a screen saver.
@@ -21,6 +21,10 @@ static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
  * Revision History:
  *
  * Changes of David Bagley <bagleyd@source.asset.com>
+ * 21-Feb-95: MANY patches from Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ * 24-Jan-95: time_displayed fixed from Chris Ross <cross@va.pubnix.com>.
+ * 18-Jan-95: Ultrix systems (at least DECstations) running enhanced
+ *            security from Chris Fuhrman <cfuhrman@vt.edu> and friend.
  * 26-Oct-94: In order to use extra-long passwords with the Linux changed
  *            PASSLENGTH to 64 <slouken@virtbrew.water.ca.gov>
  * 11-Jul-94: added -inwindow option from Greg Bowering
@@ -144,14 +148,27 @@ static char sccsid[] = "@(#)xlock.c	23.21 91/06/27 XLOCK";
 #ifndef VMS
 #include <pwd.h>
 #endif
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__hpux)
 #include <sys/types.h>
 #endif
+#ifdef ultrix
+#include <auth.h>
+#endif
+
+#ifdef HP_PASSWDETC
+#include <sys/wait.h>
+#endif
+
+#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD) || defined(SOLARIS_SHADOW)
+#define SECURE_PASSWD
+#endif
 
 #include "xlock.h"
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
+#ifdef USE_VROOT
 #include "vroot.h"
+#endif
 
 extern char *crypt();
 extern char *getenv();
@@ -168,6 +185,9 @@ long	start_time;
 static int  screens;		/* number of screens */
 static Window win[MAXSCREENS];	/* window used to cover screen */
 static Window icon[MAXSCREENS];	/* window used during password typein */
+#ifdef LOGOUT_BUTTON
+static Window button[MAXSCREENS];
+#endif
 static Window root[MAXSCREENS];	/* convenience pointer to the root window */
 static GC   textgc[MAXSCREENS];	/* graphics context used for text rendering */
 static long fgcol[MAXSCREENS];	/* used for text rendering */
@@ -186,11 +206,27 @@ static int  ssinterval;
 static int  ssblanking;
 static int  ssexposures;
 
+/* GEOMETRY STUFF */
+static int  sizeconfiguremask;
+static XWindowChanges fullsizeconfigure, minisizeconfigure;
+static int  fullscreen = False;
+static int  win_x, win_y, win_w, win_h;
+
 #define PASSLENGTH		64
 #define FALLBACK_FONTNAME	"fixed"
 #define ICONW			64
 #define ICONH			64
 
+#ifdef HP_PASSWDETC
+static char *userpass;
+static char *rootpass;
+static char *user;
+#endif
+
+#if defined(LOGOUT_BUTTON) || defined(AUTO_LOGOUT)
+static int tried_logout = 0;
+#endif
+
 #define AllPointerEventMask \
 	(ButtonPressMask | ButtonReleaseMask | \
 	EnterWindowMask | LeaveWindowMask | \
@@ -299,7 +335,7 @@ XChangeGrabbedCursor(cursor)
     Cursor      cursor;
 {
 #ifndef DEBUG
-    if (!inwindow && !(inroot && nolock))
+    if (grabmouse && !inwindow && !inroot)
       (void) XGrabPointer(dsp, win[0], True, AllPointerEventMask,
 		    GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
 #endif
@@ -313,8 +349,10 @@ static void
 finish()
 {
     XSync(dsp, False);
-    if (!nolock && inroot)
+#ifdef USE_VROOT
+    if (inroot)
         XClearWindow(dsp, win[0]);
+#endif
     if (!nolock && !allowaccess)
 	XUngrabHosts(dsp);
     XUngrabPointer(dsp, CurrentTime);
@@ -325,6 +363,180 @@ finish()
     XCloseDisplay(dsp);
 }
 
+/*
+ * Convenience function for drawing text
+ */
+static void
+putText(win, gc, string, bold, left, px, py)
+    Window win;               /* which window */
+    GC gc;            /* gc */
+    char *string;     /* text to write */
+    int bold;         /* 1 = make it bold */
+    int left;         /* left edge of text */
+    int *px, *py;     /* current x and y, input & return */
+{
+#define PT_BUFSZ 2048
+    char buf[PT_BUFSZ], *p, *s;
+    int x = *px, y = *py, last, len;
+
+    (void) strncpy(buf, string, PT_BUFSZ);
+    buf[PT_BUFSZ - 1] = 0;
+
+    p = buf;
+    last = 0;
+    while(1) {
+      s = p;
+      for(; *p; ++p)
+          if(*p == '\n')
+              break;
+      if(!*p)
+          last = 1;
+      *p = 0;
+
+      if ((len = strlen(s))) {   /* yes, that's supposed to be "=", not "==" */
+          XDrawImageString(dsp, win, gc, x, y, s, len);
+          if(bold)
+              XDrawString(dsp, win, gc, x+1, y, s, len);
+      }
+      if(!last) {
+          y += font->ascent + font->descent + 2;
+          x = left;
+      } else {
+          if(len)
+              x += XTextWidth(font, s, len);
+          break;
+      }
+      p++;
+    }
+    *px = x;
+    *py = y;
+}
+
+
+#ifndef LO_BUTTON_TIME
+#define LO_BUTTON_TIME 5
+#endif
+
+static void
+statusUpdate(new, scr)
+    Bool new;
+    int scr;
+{
+    int left, x, y, now, len;
+    char buf[1024];
+    XWindowAttributes xgwa;
+    static int last_time;
+#ifdef LOGOUT_BUTTON
+    int ysave;
+    static made_button, last_tried_lo = 0;
+    static int loButton = LO_BUTTON_TIME;
+#endif /* LOGOUT_BUTTON */
+
+    now = seconds();
+    len = (now - start_time) / 60;
+
+#ifdef LOGOUT_BUTTON
+    if (tried_logout && !last_tried_lo) {
+      last_tried_lo = 1;
+      new = 1;
+#ifdef AUTO_LOGOUT
+      forceLogout = 0;
+#endif
+    }
+
+    if (new)
+      made_button = 0;
+#endif /* LOGOUT_BUTTON */
+
+    if (new || last_time != len)
+      last_time = len;
+    else
+      return;
+
+    (void) XGetWindowAttributes(dsp, win[scr], &xgwa);
+    x = left = iconx[scr];
+    y = xgwa.height - (font->descent + 2);
+#ifdef AUTO_LOGOUT
+    if (forceLogout)
+      y -= (font->ascent + font->descent + 2);
+#endif
+#ifdef LOGOUT_BUTTON
+    if (enable_button && loButton) {
+      if (loButton > len) {
+          y -= (font->ascent + font->descent + 2);
+          (void) sprintf(buf,
+              "%d minute%s until public logout button appears.    \n",
+              loButton - len, loButton - len == 1 ? "" : "s");
+          putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+      } else if (!made_button || (new && tried_logout)) {
+          made_button = 1;
+          ysave = y;
+          y = xgwa.height * 0.6;
+
+          XUnmapWindow(dsp, button[scr]);
+          XSetForeground(dsp, Scr[scr].gc, bgcol[scr]);
+          XFillRectangle(dsp, win[scr], Scr[scr].gc, left, y,
+                      xgwa.width - left, xgwa.height - y);
+          XSetForeground(dsp, Scr[scr].gc, fgcol[scr]);
+
+          if (tried_logout) {
+              putText(win[scr], textgc[scr], logoutFailedString,
+                      True, left, &x, &y);
+          } else {
+              XMoveWindow(dsp, button[scr], left, y);
+              XMapWindow(dsp, button[scr]);
+              XRaiseWindow(dsp, button[scr]);
+              (void) sprintf(buf, " %s ", logoutButtonLabel);
+              XSetForeground(dsp, Scr[scr].gc, WhitePixel(dsp, scr));
+              XDrawString(dsp, button[scr], Scr[scr].gc, 0, font->ascent + 1,
+                          buf, strlen(buf));
+              XSetForeground(dsp, Scr[scr].gc, fgcol[scr]);
+              y += 5 + 2*font->ascent + font->descent;
+              putText(win[scr], textgc[scr], logoutButtonHelp,
+                          False, left, &x, &y);
+          }
+          y = ysave;
+          x = left;
+      }
+    }
+#endif /* LOGOUT_BUTTON */
+#ifdef AUTO_LOGOUT
+    if (forceLogout) {
+      (void) sprintf(buf, "%d minute%s until auto-logout.    \n",
+          forceLogout - len,
+          forceLogout - len == 1 ? "" : "s");
+      putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+    }
+#endif /* AUTO_LOGOUT */
+    if (timeelapsed) {
+      (void) strcpy(buf, "Display has been locked for ");
+      putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+
+      if (len < 60)
+          (void) sprintf(buf, "%d minute%s.    \n", len, len == 1 ? "" : "s");
+      else
+          (void) sprintf(buf, "%d hour%s %d minute%s.    \n", 
+                      len / 60, len / 60 == 1 ? "" : "s",
+                      len % 60, len % 60 == 1 ? "" : "s");
+      putText(win[scr], textgc[scr], buf, False, left, &x, &y);
+    }
+}
+
+#ifdef AUTO_LOGOUT
+static void
+checkLogout()
+{
+    if (nolock || tried_logout || !forceLogout)
+      return;
+
+    if (seconds() > forceLogout*60 + start_time) {
+      tried_logout = 1;
+      forceLogout = 0;
+      logoutUser();
+    }
+}
+#endif /* AUTO_LOGOUT */
+
 
 static int
 ReadXString(s, slen)
@@ -339,12 +551,14 @@ ReadXString(s, slen)
     int         len;
     int         thisscreen = screen;
     char        pwbuf[PASSLENGTH];
+    int         first_key = 1;
 
     for (screen = 0; screen < screens; screen++)
 	if (thisscreen == screen)
 	    init(icon[screen]);
 	else
 	    init(win[screen]);
+    statusUpdate(True, thisscreen);
     bp = 0;
     *s = 0;
     while (True) {
@@ -355,7 +569,8 @@ ReadXString(s, slen)
 		    callback(icon[screen]);
 		else
 		    callback(win[screen]);
-	    XFlush(dsp);
+	    statusUpdate(False, thisscreen);
+	    XSync(dsp, False);
 	    (void) usleep(delay);
 	    if (seconds() - lasteventtime > timeout) {
 		screen = thisscreen;
@@ -377,6 +592,8 @@ ReadXString(s, slen)
 		    break;
 		case 10:	/* ^J */
 		case 13:	/* ^M */
+	            if (first_key && usefirst)
+	                break;
 		    s[bp] = '\0';
 		    return 0;
 		case 21:	/* ^U */
@@ -422,6 +639,18 @@ ReadXString(s, slen)
 	    if (((XButtonEvent *) & event)->window == icon[screen]) {
 		return 1;
 	    }
+#ifdef LOGOUT_BUTTON
+	    if (((XButtonEvent *) & event)->window == button[screen]) {
+	        XSetFunction(dsp, Scr[screen].gc, GXxor);
+	        XSetForeground(dsp, Scr[screen].gc, fgcol[screen]);
+	        XFillRectangle(dsp, button[screen], Scr[screen].gc,
+	                0, 0, 500, 100);
+	        XSync(dsp, False);
+	        tried_logout = 1;
+	        logoutUser();
+	        XSetFunction(dsp, Scr[screen].gc, GXcopy);
+	    }
+#endif
 	    break;
 
 	case VisibilityNotify:
@@ -451,6 +680,7 @@ ReadXString(s, slen)
 		     ProgramName, event.type);
 	    break;
 	}
+	first_key = 0;
     }
 }
 
@@ -464,6 +694,7 @@ getPassword()
 #endif
 
     char        buffer[PASSLENGTH];
+#ifndef HP_PASSWDETC
     char        userpass[PASSLENGTH];
 #ifdef SOLARIS_SHADOW
     char      user[L_cuserid];
@@ -472,8 +703,9 @@ getPassword()
     char        rootpass[PASSLENGTH];
     char       *user;
 #endif
+#endif /* HP_PASSWDETC */
     XWindowAttributes xgwa;
-    int         y, left, done;
+    int         x, y, left, done;
 #ifdef VMS
     char *root = "SYSTEM";
  
@@ -551,16 +783,17 @@ getPassword()
     struct pr_passwd *pw;
 
     pw = getprpwnam("root");
-    strcpy(rootpass, pw->ufld.fd_encrypt);
+    (void) strcpy(rootpass, pw->ufld.fd_encrypt);
 
     pw = getprpwuid (getuid());
-    strcpy(userpass, pw->ufld.fd_encrypt);
+    (void) strcpy(userpass, pw->ufld.fd_encrypt);
     user = pw->ufld.fd_name;
 
 #else /* !OSF1_ENH_SEC */
+#ifndef HP_PASSWDETC
     struct passwd *pw;
 
-#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
+#if defined(SECURE_PASSWD)
     seteuid(0);       /* Need to be root now */
 #endif
 
@@ -569,15 +802,12 @@ getPassword()
        (void) strcpy(rootpass, pw->pw_passwd);
        pw = (struct passwd *)getpwnam(cuserid(NULL));*/
     sp = getspnam(cuserid(user));
-    strcpy(userpass, sp->sp_pwdp);
+    (void) strcpy(userpass, sp->sp_pwdp);
 #else
     pw = (struct passwd *)getpwnam("root");
     (void) strcpy(rootpass, pw->pw_passwd);
 
     pw = (struct passwd *)getpwuid(getuid());
-    /* Try this instead for HPUX */
-    /* pw = (struct passwd *)getpwnam(cuserid(NULL)); */
-
 #endif
 #if defined(HAS_SHADOW) && defined(linux)
     {
@@ -595,11 +825,17 @@ getPassword()
     user = pw->pw_name;
 #endif
 #endif /* !OFS_ENH_SEC */
-#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
+#if defined(SECURE_PASSWD)
     seteuid(getuid());                /* Don't need to be root anymore */
 #endif
+#endif /* !HP_PASSWDETC */
 #endif /* !VMS */
 
+    if ( !fullscreen )
+       XConfigureWindow( dsp, win[screen], sizeconfiguremask,
+               &fullsizeconfigure);
+
+
     (void) XGetWindowAttributes(dsp, win[screen], &xgwa);
 
     XChangeGrabbedCursor(XCreateFontCursor(dsp, XC_left_ptr));
@@ -611,46 +847,24 @@ getPassword()
     XMapWindow(dsp, icon[screen]);
     XRaiseWindow(dsp, icon[screen]);
 
-    left = iconx[screen] + ICONW + font->max_bounds.width;
+    x = left = iconx[screen] + ICONW + font->max_bounds.width;
     y = icony[screen] + font->ascent;
 
-    XDrawString(dsp, win[screen], textgc[screen],
-		left, y, text_name, strlen(text_name));
-    XDrawString(dsp, win[screen], textgc[screen],
-		left + 1, y, text_name, strlen(text_name));
-    XDrawString(dsp, win[screen], textgc[screen],
-		left + XTextWidth(font, text_name, strlen(text_name)), y,
-		user, strlen(user));
+    putText(win[screen], textgc[screen], text_name, True, left, &x, &y);
+    putText(win[screen], textgc[screen], user, False, left, &x, &y);
+    putText(win[screen], textgc[screen], "\n", False, left, &x, &y);
 
-    y += font->ascent + font->descent + 2;
-    XDrawString(dsp, win[screen], textgc[screen],
-		left, y, text_pass, strlen(text_pass));
-    XDrawString(dsp, win[screen], textgc[screen],
-		left + 1, y, text_pass, strlen(text_pass));
+    putText(win[screen], textgc[screen], text_pass, True, left, &x, &y);
+    putText(win[screen], textgc[screen], " ", False, left, &x, &y);
 
-    passx = left + 1 + XTextWidth(font, text_pass, strlen(text_pass))
-	+ XTextWidth(font, " ", 1);
+    passx = x;
     passy = y;
 
     y = icony[screen] + ICONH + font->ascent + 2;
-    XDrawString(dsp, win[screen], textgc[screen],
-		iconx[screen], y, text_info, strlen(text_info));
+    x = left = iconx[screen];
+    putText(win[screen], textgc[screen], text_info, False, left, &x, &y);
+    putText(win[screen], textgc[screen], "\n", False, left, &x, &y);
 
-    if (timeelapsed) {
-      char clock[16];
-      int day, hour, minute, second;
-      long time_displayed = seconds() - start_time;
-      if (time_displayed < 0)
-        time_displayed = 0;
-      day = (int) (time_displayed / (24 * 60 * 60));
-      hour = (int) ((time_displayed / (24 * 60)) % 60);
-      minute = (int) ((time_displayed / 60) % 60);
-      second = (int) (time_displayed % 60);
-      (void) sprintf (clock, "%d:%02d:%02d:%02d", day, hour, minute, second);
-      y += font->ascent + font->descent + 2;
-      XDrawString(dsp, win[screen], textgc[screen],
-		iconx[screen], y, clock, strlen(clock));
-    }
     XFlush(dsp);
 
     y += font->ascent + font->descent + 2;
@@ -684,12 +898,18 @@ getPassword()
         done = !((strcmp(hash_password,hash_system)) &&
                  (!allowroot || strcmp(root_password,root_system)));
 #else /* !VMS */
+#ifdef ultrix
+        done = ((authenticate_user(pw, buffer, NULL) >= 0) ||
+        (allowroot && (authenticate_user((struct passwd *) getpwnam("root"),
+                       buffer, NULL) >= 0)));
+#else
 #if defined(HAS_SHADOW) && defined(linux)
 #ifdef __QNX__
 #define pw_encrypt crypt
 #endif
-        done = !((strcmp(pw_encrypt(buffer, userpass), userpass)) &&
-          (!allowroot || strcmp(pw_encrypt(buffer, rootpass), rootpass)));
+        done = !((strcmp((char *)pw_encrypt(buffer, userpass), userpass)) &&
+                 (!allowroot ||
+                  strcmp((char *)pw_encrypt(buffer, rootpass), rootpass)));
 #else
 #ifdef SOLARIS_SHADOW
         done = !((strcmp(crypt(buffer, userpass), userpass)));
@@ -698,11 +918,13 @@ getPassword()
           (!allowroot || strcmp(crypt(buffer, rootpass), rootpass)));
 #endif
 #endif
+#endif
 #endif /* !VMS */
 	if (!done && !*buffer) {
 	    /* just hit return, and it wasn't his password */
 	    break;
 	}
+#ifndef ultrix
 	if (!*userpass && *buffer) {
 	    /*
 	     * the user has no password, but something was typed anyway.
@@ -710,6 +932,7 @@ getPassword()
 	     */
 	    done = False;
 	}
+#endif
 	/* clear plaintext password so you can't grunge around /dev/kmem */
 	(void) memset(buffer, 0, sizeof(buffer));
 
@@ -723,9 +946,12 @@ getPassword()
 	XDrawString(dsp, win[screen], textgc[screen],
 		    iconx[screen], y, text_valid, strlen(text_valid));
 
-	if (done)
+	if (done) {
+           if ( !fullscreen )
+               XConfigureWindow( dsp, win[screen], sizeconfiguremask,
+                       &minisizeconfigure );
 	    return 0;
-	else {
+	} else {
 	    XSync(dsp, True);	/* flush input buffer */
 	    (void) sleep(1);
 	    XFillRectangle(dsp, win[screen], Scr[screen].gc,
@@ -743,6 +969,12 @@ getPassword()
     }
     XChangeGrabbedCursor(mycursor);
     XUnmapWindow(dsp, icon[screen]);
+#ifdef LOGOUT_BUTTON
+    XUnmapWindow(dsp, button[screen]);
+#endif
+    if ( !fullscreen )
+        XConfigureWindow( dsp, win[screen], sizeconfiguremask,
+               &minisizeconfigure );
     return 1;
 }
 
@@ -755,11 +987,14 @@ justDisplay()
     for (screen = 0; screen < screens; screen++)
 	init(win[screen]);
     do {
-	while (!XPending(dsp)) {
+        while (!XPending(dsp)) {
+            (void) usleep(delay);
 	    for (screen = 0; screen < screens; screen++)
 		callback(win[screen]);
-	    XFlush(dsp);
-	    (void) usleep(delay);
+#ifdef AUTO_LOGOUT
+	    checkLogout();
+#endif
+	    XSync(dsp, False);
 	}
 	XNextEvent(dsp, &event);
 #ifndef DEBUG
@@ -774,6 +1009,11 @@ justDisplay()
     for (screen = 0; screen < screens; screen++)
 	if (event.xbutton.root == RootWindow(dsp, screen))
 	    break;
+
+    /* KLUDGE SO TVTWM AND VROOT WON'T MAKE XLOCK DIE */
+    if (screen >= screens)
+         screen = 0;
+
     if (usefirst)
 	XPutBackEvent(dsp, &event);
 }
@@ -824,13 +1064,13 @@ lockDisplay()
 	(void) sigsetmask(oldsigmask);
 #endif
     }
-#ifdef __hpux
+#if defined(__hpux) || defined(__apollo)
 	XHPDisableReset(dsp);
 #endif
     do {
 	justDisplay();
     } while (getPassword());
-#ifdef __hpux
+#if defined(__hpux) || defined(__apollo)
 	XHPEnableReset(dsp);
 #endif
 }
@@ -854,6 +1094,99 @@ allocpixel(cmap, name, def)
 }
 
 
+#ifdef HP_PASSWDETC
+/*
+ * The PasswdEtc libraries have replacement passwd functions that make
+ * queries to DomainOS registries.  Unfortunately, these functions do
+ * wierd things to the process (at minimum, signal handlers get changed,
+ * there are probably other things as well) that cause xlock to become
+ * unstable.
+ *
+ * As a (really, really sick) workaround, we'll fork() and do the getpw*()
+ * calls in the child, and pass the information back through a pipe.
+ */
+void get_passwords()
+{
+    struct passwd *pw;
+    int pipefd[2], n, total = 0, stat_loc;
+    static char buf[1024];    /* should be big enough */
+    pid_t pid;
+
+    pipe(pipefd);
+
+    if((pid = fork()) == 0) {
+      close(pipefd[0]);
+      pw = getpwuid(getuid());
+      write(pipefd[1], pw->pw_name, strlen(pw->pw_name));
+      write(pipefd[1], "\n", 1);
+      write(pipefd[1], pw->pw_passwd, strlen(pw->pw_passwd));
+      write(pipefd[1], "\n", 1);
+      pw = getpwnam("root");
+      write(pipefd[1], pw->pw_passwd, strlen(pw->pw_passwd));
+      close(pipefd[1]);
+      _exit(0);
+    }
+    if(pid < 0) {
+      (void) fprintf(stderr, "Could not get passwords (fork failed)\n");
+      exit(1);
+    }
+    close(pipefd[1]);
+
+    while((n = read(pipefd[0], &(buf[total]), 50)) > 0)
+      total += n;
+
+    wait(&stat_loc);
+
+    if(n < 0) {
+      (void) fprintf(stderr, "Could not get passwords (read failed)\n");
+      exit(1);
+    }
+    buf[total] = 0;
+
+    if(total < 3) {
+      (void) fprintf(stderr, "Could not get passwords (lookups failed)\n");
+      exit(1);
+    }
+
+    user = strtok(buf, "\n");
+    userpass = strtok(NULL, "\n");
+    rootpass = strtok(NULL, "\n");
+
+    if(!user || !userpass || !rootpass) {
+      (void) fprintf(stderr, "Could not get passwords (lookups failed 2)\n");
+      exit(1);
+    }
+}
+#endif        /* HP_PASSWDETC */
+
+void
+set_colormap(screen, map)
+    int screen;
+    Colormap map;
+{
+    Screen *scr = ScreenOfDisplay(dsp, screen);
+
+    if(inroot || mono || CellsOfScreen(scr) == 2)
+      return;
+
+    XSetWindowColormap(dsp, win[screen], map);
+    /*
+     * Now, here we have a problem.  When we're running full-screen, the
+     * window's override_redirect attribute is on.  So, the window manager
+     * never gets the ColormapNotify event that gets generated on the above
+     * XSetWindowColormap() call, and doesn't install the new colormap.
+     * So, a quick solution is to install it ourselves.  The problem with
+     * this is that it violates the ICCCM convention that only window
+     * managers should install colormaps.  Indeed, Fvwm _enforces_ this
+     * by immediately un-doing any XInstallColormap() performed by a client
+     * (which is why this doesn't work right under Fvwm).
+     */
+    if(!inroot && !inwindow) {
+      XInstallColormap(dsp, map);
+    }
+}
+
+
 int
 main(argc, argv)
     int         argc;
@@ -863,6 +1196,10 @@ main(argc, argv)
     XGCValues   xgcv;
     XColor      nullcolor;
 
+#ifdef HP_PASSWDETC
+    get_passwords();
+#endif
+
     ProgramName = strrchr(argv[0], '/');
     if (ProgramName)
 	ProgramName++;
@@ -873,12 +1210,12 @@ main(argc, argv)
     set_auth_parameters();
 #endif
 
-#if defined(__NetBSD__) || defined(__QNX__) || defined(HPUX_SECURE_PASSWD)
+#if defined(SECURE_PASSWD)
     seteuid(getuid());
 #endif
 
     start_time = time((long *) 0);
-    srandom(start_time);	/* random mode needs the seed set. */
+    SRAND(start_time);	/* random mode needs the seed set. */
 
     GetResources(argc, argv);
 
@@ -892,12 +1229,28 @@ main(argc, argv)
 	if (font == NULL)
 	    error("%s: can't even find %s!!!\n", FALLBACK_FONTNAME);
     }
+    {
+      int flags, x, y;
+      unsigned int w, h;
+
+      if ( *geometry == '\0' ) {
+          fullscreen = True;
+      }
+      else {
+          flags = XParseGeometry( geometry, &x, &y, &w, &h );
+          win_x = minisizeconfigure.x = flags&XValue ? x : 0;
+          win_y = minisizeconfigure.y = flags&YValue ? y : 0;
+          win_w = minisizeconfigure.width = flags&WidthValue ? w : ICONW;
+          win_h = minisizeconfigure.height = flags&HeightValue ? h : ICONH;
+      }
+    }
     screens = ScreenCount(dsp);
     if (screens > MAXSCREENS)
 	error("%s: can only support %d screens.\n", MAXSCREENS);
     for (screen = 0; screen < screens; screen++) {
 	Screen     *scr = ScreenOfDisplay(dsp, screen);
 	Colormap    cmap = DefaultColormapOfScreen(scr);
+	Colormap    dcmap = cmap;        /* record default colormap */
 
 	root[screen] = RootWindowOfScreen(scr);
         bgcol[screen] = allocpixel(cmap, background, "White");
@@ -915,21 +1268,37 @@ main(argc, argv)
 	    u_char      green[NUMCOLORS];
 	    u_char      blue[NUMCOLORS];
 	    int         i;
+	    XColor      xcolor;
 
+	    if (install) {
+		/* "allocate" the black and white pixels, so that they'll be
+		   included by XCopyColormapAndFree() if it gets called */
+		xcolor.pixel = BlackPixelOfScreen(scr);
+		XQueryColor(dsp, cmap, &xcolor);
+		XAllocColor(dsp, cmap, &xcolor);
+		xcolor.pixel = WhitePixelOfScreen(scr);
+		XQueryColor(dsp, cmap, &xcolor);
+		XAllocColor(dsp, cmap, &xcolor);
+	    }
 	    hsbramp(0.0, saturation, 1.0, 1.0, saturation, 1.0, colorcount,
 		    red, green, blue);
 	    Scr[screen].npixels = 0;
 	    for (i = 0; i < colorcount; i++) {
-		XColor      xcolor;
-
 		xcolor.red = red[i] << 8;
 		xcolor.green = green[i] << 8;
 		xcolor.blue = blue[i] << 8;
 		xcolor.flags = DoRed | DoGreen | DoBlue;
 
-		if (!XAllocColor(dsp, cmap, &xcolor))
-		    break;
-
+		if (!XAllocColor(dsp, cmap, &xcolor)) {
+		    if(!install || cmap != dcmap)
+			break;
+		    if((cmap = XCopyColormapAndFree(dsp, cmap)) == dcmap)
+			break;
+		    if(verbose)
+			fprintf(stderr, "using private colormap\n");
+		    if(!XAllocColor(dsp, cmap, &xcolor))
+			break;
+		}
 		Scr[screen].pixels[i] = xcolor.pixel;
 		Scr[screen].npixels++;
 	    }
@@ -939,7 +1308,7 @@ main(argc, argv)
 	}
 
 	xswa.override_redirect = True;
-	xswa.background_pixel = BlackPixelOfScreen(scr);
+	xswa.background_pixel = BlackPixel(dsp, screen);
 	xswa.event_mask = KeyPressMask | ButtonPressMask |
 #ifdef MOUSEMOTION
                           MotionNotify |
@@ -952,15 +1321,34 @@ main(argc, argv)
 #else
 #define WIDTH (inwindow? WidthOfScreen(scr)/2 : WidthOfScreen(scr))
 #define HEIGHT (inwindow? HeightOfScreen(scr)/2 : HeightOfScreen(scr))
-#define CWMASK (((inwindow || (inroot && nolock))? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
+#define CWMASK (((inwindow||inroot)? 0 : CWOverrideRedirect) | CWBackPixel | CWEventMask)
 #endif
+#ifdef USE_VROOT
        if (inroot) {
             win[screen] = root[screen];
             XChangeWindowAttributes(dsp, win[screen], CWBackPixel, &xswa);
-        } else {
-	    win[screen] = XCreateWindow(dsp, root[screen], 0, 0, WIDTH, HEIGHT, 0,
-				 CopyFromParent, InputOutput, CopyFromParent,
-				    CWMASK, &xswa);
+       } else
+#endif
+       {
+            if ( fullscreen )
+                win[screen] = XCreateWindow(dsp, root[screen],
+                               0, 0, WIDTH, HEIGHT, 0,
+                               CopyFromParent, InputOutput, CopyFromParent,
+                               CWMASK, &xswa);
+            else {
+                sizeconfiguremask = CWX | CWY | CWWidth | CWHeight;
+                fullsizeconfigure.x = 0;
+                fullsizeconfigure.y = 0;
+                fullsizeconfigure.width = WIDTH;
+                fullsizeconfigure.height = HEIGHT;
+                win[screen] = XCreateWindow(dsp, root[screen],
+                               (int) minisizeconfigure.x,
+                               (int) minisizeconfigure.y,
+                               (unsigned int) minisizeconfigure.width,
+                               (unsigned int) minisizeconfigure.height,
+                               0, CopyFromParent, InputOutput, CopyFromParent,
+                                        CWMASK, &xswa);
+            }
         }
 
 #ifdef DEBUG
@@ -990,8 +1378,8 @@ main(argc, argv)
 
 	icony[screen] = DisplayHeight(dsp, screen) / 6;
 
-	xswa.border_pixel = fgcol[screen];
-	xswa.background_pixel = bgcol[screen];
+	xswa.border_pixel = WhitePixel(dsp, screen);
+	xswa.background_pixel = BlackPixel(dsp, screen);
 	xswa.event_mask = ButtonPressMask;
 #define CIMASK CWBorderPixel | CWBackPixel | CWEventMask
 	icon[screen] = XCreateWindow(dsp, win[screen],
@@ -1000,17 +1388,34 @@ main(argc, argv)
 				     InputOutput, CopyFromParent,
 				     CIMASK, &xswa);
 
+#ifdef LOGOUT_BUTTON
+	{
+	    char buf[1024];
+	    int w, h;
+
+	    (void) sprintf(buf, " %s ", logoutButtonLabel);
+	    w = XTextWidth(font, buf, strlen(buf));
+	    h = font->ascent + font->descent + 2;
+	    button[screen] = XCreateWindow(dsp, win[screen],
+				0, 0, w, h, 1, CopyFromParent,
+				InputOutput, CopyFromParent,
+				CIMASK, &xswa);
+	}
+#endif
 	XMapWindow(dsp, win[screen]);
 	XRaiseWindow(dsp, win[screen]);
 
-	xgcv.foreground = WhitePixelOfScreen(scr);
-	xgcv.background = BlackPixelOfScreen(scr);
+	if(install && cmap != dcmap)
+	    set_colormap(screen, cmap);
+
+	xgcv.font = font->fid;
+	xgcv.foreground = WhitePixel(dsp, screen);
+	xgcv.background = BlackPixel(dsp, screen);
 	Scr[screen].gc = XCreateGC(dsp, win[screen],
-				   GCForeground | GCBackground, &xgcv);
+				GCFont | GCForeground | GCBackground, &xgcv);
 
 	xgcv.foreground = fgcol[screen];
 	xgcv.background = bgcol[screen];
-	xgcv.font = font->fid;
 	textgc[screen] = XCreateGC(dsp, win[screen],
 				GCFont | GCForeground | GCBackground, &xgcv);
     }
@@ -1028,12 +1433,13 @@ main(argc, argv)
 	XSetScreenSaver(dsp, 0, 0, 0, 0);	/* disable screen saver */
     }
 #ifndef DEBUG
-    if (!inwindow && !(inroot && nolock))
-      GrabKeyboardAndMouse();
+    if (grabmouse && !inwindow && !inroot)
+       GrabKeyboardAndMouse();
+    else
+       nolock = 1;
 #endif
 
     (void) nice(nicelevel);
-
     if (nolock)
 	justDisplay();
     else
